function [U,cmap,cbar] = color_skeleton(B,S,D)
%COLOR_SKELETON   Colorize binarization, skeletonization and distances
%   [U,CMAP,CBAR] = COLOR_SKELETON(B,S,D) produces an indexed 
%   image and a corresponding colormap for visualization of binarization, 
%   skeletonization and distances.
%
%   B - binary volume (zero outsice, one inside)
%   S - binary volume for skeleton of both inside and outside
%   D - signed distance field (negative for inside, positive for outside)
%   U - indexed uint8 image
%   CMAP - associated colormap
%   CBAR - struct containing inputs for associated colorbar
%   
%   Example:
%       [U,cmap,cbar] = color_skeleton(B,S,D);
%       imagesc(U)
%       colormap(cmap)
%       colorbar(cbar{:})
%
%   Copyright 2018 Vedrana Andersen Dahl, vand@dtu.dk 


% Colormap values correspond to
% 0 - outside (black)
% 1 - inside (white)
% 2--128 positive (outside) distance (127 colors from winter colormap)
% 129--255 negative (inside) distance (127 colors from autumn colormap)

U = uint8(B);
ds = D(S);
minmax = [min(ds),max(ds)]; % min should be negative
ds(ds>0) = 2+126*(minmax(2)-ds(ds>0))/minmax(2); % flipping for symmetry
ds(ds<0) = 129+126*ds(ds<0)/minmax(1);
U(S) = ds;
cmap = [0 0 0; 1 1 1; autumn(127);flip(winter(127))];
cbtl = round(10*[minmax(2),0,-minmax(1)])/10; % colorbar tick labels
cbar = {'Limits',[2,255],'Ticks',[2,128.5,255],'TickLabels',cbtl};
